using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Xml;
using UnityEngine;

public class DeserializedLevelsLoader
{
    // Levels deserialized
    private DeserializedLevels deserializedLevels;

    private int currentLevelIndex = 0;
    private int currentSegmentIndex = 0;

    private const string prefabsFolder = "Prefabs/";

    struct ItemStruct
    {
        public GameObject prefab;
        public float x;
        public float y;
        public float rot;
        public float scalex;
        public float scaley;
    }

    // Cache prefabs in prefabDict
    Dictionary<string,GameObject> prefabPool;

    // Cache all items with locations
    List<ItemStruct> sceneItemsList;

    Transform parentOfXmlItems;
    
    public const string xmlItemsGOName = "XmlItems";

    public void loadLevel (int levelIndex = 0)
    {
        prefabPool = new Dictionary<string, GameObject>();
        sceneItemsList = new List<ItemStruct>();

        // if the XmlItems gameobject folder remained in the Hierarcy, then delete it
        while (GameObject.Find (xmlItemsGOName) != null)
            MonoBehaviour.DestroyImmediate(GameObject.Find (xmlItemsGOName));
        
        parentOfXmlItems = new GameObject(xmlItemsGOName).transform;

        deserializedLevels = XmlIO.LoadXml<DeserializedLevels>("Levels");

        // if startlevel is in the XML i.e. <Developer StartLevel="3" /> then get level from there
        // otherwise start with level 1
         currentLevelIndex = (levelIndex > deserializedLevels.levels.Length - 1) ? 0 : levelIndex;


        DeserializedLevels.Level currentLevel = deserializedLevels.levels[currentLevelIndex];
        sceneItemsList.Clear();

     
        // <Item prefab="Chair" x="1" y="10" rot="90" />
        foreach (DeserializedLevels.Item deserializedItem in currentLevel.items)
        {
            // caching prefabString i.e. "phone"
            string prefabString = deserializedItem.prefab;

            // if the prefab in the item XmlNode has not been loaded then add it to the prefabsDict dictionary,
            if (!prefabPool.ContainsKey(prefabString))
            {
                // load prefab
                GameObject prefabObject = Resources.Load(prefabsFolder + prefabString, typeof(GameObject)) as GameObject;

                // if unsuccesful, error message and jump to next in the foreach loop
                if (prefabObject == null)
                {
                    Debug.LogError("Prefab \"" + prefabString + "\" does not exists.");
                    continue;
                }

                // otherwise add to dictionary
                prefabPool.Add(prefabString, prefabObject);
            }

            ItemStruct item;
            item.prefab = prefabPool[prefabString];
            item.x = toFloatZeroIfNull(deserializedItem.x);
            item.y = toFloatZeroIfNull(deserializedItem.y);
            item.rot = toFloatZeroIfNull(deserializedItem.rot);
            item.scalex = toFloatOneIfNull(deserializedItem.scalex);
            item.scaley = toFloatOneIfNull(deserializedItem.scaley);

            sceneItemsList.Add(item);
        }

        // Finally instantiate all items
        foreach (ItemStruct item in sceneItemsList)
        {

            // TODO load height coordinate from a directory
            GameObject newGameObject = MonoBehaviour.Instantiate(item.prefab) as GameObject;

            // set position
            setPos2D(newGameObject, new Vector2(item.x, item.y));

            // set rotation
            setRot2D(newGameObject, item.rot);

            setScale2D(newGameObject, item.scalex, item.scaley);

            // set parent
            newGameObject.transform.parent = parentOfXmlItems;
        }
    }


    // DONE, these are only helper functions below

    // if no value then return zero or one, otherwise convert to float
    float toFloatZeroIfNull (string value) { return value == null ? 0 : float.Parse(value);    }
    float toFloatOneIfNull  (string value) { return value == null ? 1 : float.Parse(value);    }

    void setPos2D(GameObject g, Vector2 pos)
    {
        g.transform.position = new Vector2 (
            pos.x,
            pos.y
        );
    }

    void setRot2D(GameObject g, float rot)
    {
        //Debug.Log("Set rotation of " + g.name + " to " + rot);
        Quaternion rotation = Quaternion.identity;
        rotation.eulerAngles = new Vector3(0, 0, rot);
        g.transform.localRotation = rotation;
    }

    void setScale2D(GameObject g, float scaleX, float scaleY)
    {
        g.transform.localScale = new Vector2(scaleX, scaleY);
    }
}
